import dispatcher from 'dispatcher';

var pointermap = dispatcher.pointermap;
// radius around touchend that swallows mouse events
var DEDUP_DIST = 25;

var WHICH_TO_BUTTONS = [0, 1, 4, 2];

var HAS_BUTTONS = false;
try {
  HAS_BUTTONS = new MouseEvent('test', {buttons: 1}).buttons === 1;
} catch (e) {}

// handler block for native mouse events
var mouseEvents = {
  POINTER_ID: 1,
  POINTER_TYPE: 'mouse',
  events: [
    'mousedown',
    'mousemove',
    'mouseup',
    'mouseover',
    'mouseout'
  ],
  register: function(target) {
    dispatcher.listen(target, this.events);
  },
  unregister: function(target) {
    dispatcher.unlisten(target, this.events);
  },
  lastTouches: [],
  // collide with the global mouse listener
  isEventSimulatedFromTouch: function(inEvent) {
    var lts = this.lastTouches;
    var x = inEvent.clientX, y = inEvent.clientY;
    for (var i = 0, l = lts.length, t; i < l && (t = lts[i]); i++) {
      // simulated mouse events will be swallowed near a primary touchend
      var dx = Math.abs(x - t.x), dy = Math.abs(y - t.y);
      if (dx <= DEDUP_DIST && dy <= DEDUP_DIST) {
        return true;
      }
    }
  },
  prepareEvent: function(inEvent) {
    var e = dispatcher.cloneEvent(inEvent);
    // forward mouse preventDefault
    var pd = e.preventDefault;
    e.preventDefault = function() {
      inEvent.preventDefault();
      pd();
    };
    e.pointerId = this.POINTER_ID;
    e.isPrimary = true;
    e.pointerType = this.POINTER_TYPE;
    if (!HAS_BUTTONS) {
      e.buttons = WHICH_TO_BUTTONS[e.which] || 0;
    }
    return e;
  },
  mousedown: function(inEvent) {
    if (!this.isEventSimulatedFromTouch(inEvent)) {
      var p = pointermap.has(this.POINTER_ID);
      // TODO(dfreedman) workaround for some elements not sending mouseup
      // http://crbug/149091
      if (p) {
        this.cancel(inEvent);
      }
      var e = this.prepareEvent(inEvent);
      pointermap.set(this.POINTER_ID, inEvent);
      dispatcher.down(e);
    }
  },
  mousemove: function(inEvent) {
    if (!this.isEventSimulatedFromTouch(inEvent)) {
      var e = this.prepareEvent(inEvent);
      dispatcher.move(e);
    }
  },
  mouseup: function(inEvent) {
    if (!this.isEventSimulatedFromTouch(inEvent)) {
      var p = pointermap.get(this.POINTER_ID);
      if (p && p.button === inEvent.button) {
        var e = this.prepareEvent(inEvent);
        dispatcher.up(e);
        this.cleanupMouse();
      }
    }
  },
  mouseover: function(inEvent) {
    if (!this.isEventSimulatedFromTouch(inEvent)) {
      var e = this.prepareEvent(inEvent);
      dispatcher.enterOver(e);
    }
  },
  mouseout: function(inEvent) {
    if (!this.isEventSimulatedFromTouch(inEvent)) {
      var e = this.prepareEvent(inEvent);
      dispatcher.leaveOut(e);
    }
  },
  cancel: function(inEvent) {
    var e = this.prepareEvent(inEvent);
    dispatcher.cancel(e);
    this.cleanupMouse();
  },
  cleanupMouse: function() {
    pointermap['delete'](this.POINTER_ID);
  }
};

export default mouseEvents;
